#!/usr/bin/env python
# -*- coding: iso-8859-15 -*- 

import dbus, gtk, sys

class MainWindow:
	# get new command from entry; run; clear
	def send_command(self, widget, entry):
		entry_text = entry.get_text()
		if entry_text == "":
			return
		entry.set_text("")
		answer = self.api_object.send_message(entry_text)	 

	def write_textbuffer(self, message):
		enditer = self.textbuffer.get_end_iter()
		mark = self.textbuffer.create_mark(None, enditer, False)
		self.textbuffer.insert(enditer, message + '\n')
		self.textview.scroll_to_mark(mark, 0, True, 0.0, 1.0)
		self.textbuffer.delete_mark(mark)

	def __init__(self):
		
		window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		window.set_resizable(True)
		window.set_title("dbus test application")
		window.connect("delete_event", lambda w,e: gtk.main_quit())

		table = gtk.Table(2, 2, False)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		window.add(table)
		table.show()
		
		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
		self.textview = gtk.TextView()
		self.textbuffer = self.textview.get_buffer()
		self.textview.set_editable(False)
		self.textview.set_wrap_mode(gtk.WRAP_WORD)
		sw.add(self.textview)
		table.attach(sw, 0,2,0,1)
		sw.show()
		self.textview.show()

		# command entry
		entry = gtk.Entry()
		entry.set_width_chars(66)
		table.attach(entry, 0, 1, 1, 2, xoptions=gtk.FILL, yoptions=gtk.FILL)
		entry.show()
		entry.grab_focus()
		
		# send button
		button = gtk.Button("Send")
		button.connect("clicked", self.send_command, entry)
		table.attach(button, 1, 2, 1, 2, xoptions=gtk.FILL, yoptions=gtk.FILL)
		button.set_flags(gtk.CAN_DEFAULT)
		button.grab_default()
		button.show()
		window.show()

		#objects
		self.api_object = Api_obj(self)
		self.callback_obj = Callback_obj(dbus.Service("com.Skype.API", dbus.SystemBus()), self)

class Callback_obj(dbus.Object):
	def __init__(self, service, mw):
		self.mainwindow = mw
		dbus.Object.__init__(self, "/com/Skype/Client", service, [self.Notify])
	
	# Skype -> Client messages
	def Notify(self, message, message_text):
		self.mainwindow.write_textbuffer(message_text)

class Api_obj:
	def __init__(self, mw):
		self.mainwindow = mw
		
		remote_bus = dbus.SystemBus()
		
		system_service_list = remote_bus.get_service('org.freedesktop.DBus').get_object('/org/freedesktop/DBus', 'org.freedesktop.DBus').ListServices()
		skype_api_found = 0

		for service in system_service_list:
			if service=='com.Skype.API':
				skype_api_found = 1
				break

		if not skype_api_found:
			sys.exit('No running API-capable Skype found')

		skype_service = remote_bus.get_service('com.Skype.API')
		self.skype_api_object = skype_service.get_object('/com/Skype','com.Skype.API')

		answer = self.send_message('NAME SkypeApiPythonTestClient')
		if answer != 'OK':
			sys.exit('Could not bind to Skype client')

		answer = self.send_message('PROTOCOL 1')
		if answer != 'PROTOCOL 1':
			sys.exit('This test program only supports Skype API protocol version 1')

	# Client -> Skype
	def send_message(self, message):		
		self.mainwindow.write_textbuffer(message)
		answer = self.skype_api_object.Invoke(message)
		self.mainwindow.write_textbuffer(answer)
		return answer

def main():
	MainWindow()
	gtk.main()
	return 0	

if __name__ == "__main__":
	main()
